import android.annotation.SuppressLint
import android.content.ContentValues
import android.content.Context
import android.database.Cursor
import android.database.sqlite.SQLiteDatabase
import android.database.sqlite.SQLiteOpenHelper


class DatabaseHelper(context: Context) :
    SQLiteOpenHelper(context, DATABASE_NAME, null, DATABASE_VERSION) {
    companion object {
        private const val DATABASE_NAME = "myDatabase.db"
        private const val DATABASE_VERSION = 1

        private const val TABLE_ACTIVITY = "activity"
        private const val TABLE_ACTIVITY_POINTS = "tableActivityPoints"


        private const val COLUMN_ID = "id"
        private const val COLUMN_NAME = "name"
        private const val COLUMN_CONFIDENCE = "confidence"
        private const val COLUMN_LAST_UP_DATE = "lastUpdatedTime"

        // Add more columns if needed
        private const val COLUMN_VEHICLE_POINTS = "invehiclepoints"
        private const val COLUMN_BICYCLE_POINTS = "onbicyclepoints"
        private const val COLUMN_RUNNING_POINTS = "runningpoints"
        private const val COLUMN_WALKING_POINTS = "walkingpoints"
        private const val COLUMN_STILL_POINTS = "stillpoints"
    }


    fun addActivity(name: String, confidence: String, lastUpdatedTime: String) {
        val db = this.writableDatabase
        val values = ContentValues().apply {
            put(COLUMN_ID, System.currentTimeMillis().toString())
            put(COLUMN_NAME, name)
            put(COLUMN_CONFIDENCE, confidence)
            put(COLUMN_LAST_UP_DATE, lastUpdatedTime)
        }
        db.insert(TABLE_ACTIVITY, null, values)
        db.close()
    }

    @SuppressLint("Range")
    fun getActivities(): List<String> {
        val records = mutableListOf<String>()
        val db = this.readableDatabase
        val cursor: Cursor? = db.rawQuery("SELECT * FROM $TABLE_ACTIVITY", null)
        cursor?.use {
            while (cursor.moveToNext()) {
                val name = cursor.getString(cursor.getColumnIndex(COLUMN_NAME))
                val confidence = cursor.getString(cursor.getColumnIndex(COLUMN_CONFIDENCE))
                val lastUpdatedTime = cursor.getString(cursor.getColumnIndex(COLUMN_LAST_UP_DATE))
                records.add(
                    "get database from android native =====>$name - Confidence: $confidence, Last Updated " +
                            "Time: $lastUpdatedTime"
                )
            }
        }
        cursor?.close()
        db.close()
        return records
    }

    fun addActivityPoints(
        vehiclePoints: Int,
        bicyclePoints: Int,
        runningPoints: Int,
        walkingPoints: Int,
        stillPoints: Int,
    ) {
        val db = this.writableDatabase
        val values = ContentValues().apply {
            put(COLUMN_ID, System.currentTimeMillis().toString())
            put(COLUMN_VEHICLE_POINTS, vehiclePoints)
            put(COLUMN_BICYCLE_POINTS, bicyclePoints)
            put(COLUMN_RUNNING_POINTS, runningPoints)
            put(COLUMN_WALKING_POINTS, walkingPoints)
            put(COLUMN_STILL_POINTS, stillPoints)
        }
        db.insert(TABLE_ACTIVITY_POINTS, null, values)
        db.close()
    }

    fun editActivityPoints(
        id: String,
        vehiclePoints: Int,
        bicyclePoints: Int,
        runningPoints: Int,
        walkingPoints: Int,
        stillPoints: Int
    ) {
        val db = this.writableDatabase
        val values = ContentValues().apply {
            put(COLUMN_VEHICLE_POINTS, vehiclePoints)
            put(COLUMN_BICYCLE_POINTS, bicyclePoints)
            put(COLUMN_RUNNING_POINTS, runningPoints)
            put(COLUMN_WALKING_POINTS, walkingPoints)
            put(COLUMN_STILL_POINTS, stillPoints)
        }

        // Define the WHERE clause to update the specific record based on its ID
        val selection = "$COLUMN_ID = ?"
        val selectionArgs = arrayOf(id)

        // Update the record
        db.update(TABLE_ACTIVITY_POINTS, values, selection, selectionArgs)

        db.close()
    }



    @SuppressLint("Range")
    fun getAllActivityPoints(): List<ActivityPoints> {
        val activityPointsList = mutableListOf<ActivityPoints>()
        val query = "SELECT * FROM $TABLE_ACTIVITY_POINTS"
        val db = this.readableDatabase

        val cursor = db.rawQuery(query, null)
        cursor.use { cursorItem ->
            while (cursorItem.moveToNext()) {
                val id = cursorItem.getString(cursorItem.getColumnIndex(COLUMN_ID))
                val vehiclePoints = cursorItem.getInt(cursorItem.getColumnIndex(COLUMN_VEHICLE_POINTS))
                val bicyclePoints = cursorItem.getInt(cursorItem.getColumnIndex(COLUMN_BICYCLE_POINTS))
                val runningPoints = cursorItem.getInt(cursorItem.getColumnIndex(COLUMN_RUNNING_POINTS))
                val walkingPoints = cursorItem.getInt(cursorItem.getColumnIndex(COLUMN_WALKING_POINTS))
                val stillPoints = cursorItem.getInt(cursorItem.getColumnIndex(COLUMN_STILL_POINTS))

                val activityPoints = ActivityPoints(
                    id,
                    vehiclePoints,
                    bicyclePoints,
                    runningPoints,
                    walkingPoints,
                    stillPoints
                )
                activityPointsList.add(activityPoints)
            }
        }

        db.close()
        return activityPointsList
    }



    override fun onCreate(p0: SQLiteDatabase?) {
        TODO("Not yet implemented")
    }

    override fun onUpgrade(p0: SQLiteDatabase?, p1: Int, p2: Int) {
        TODO("Not yet implemented")
    }
}
